<?php
/*
Plugin Name: Splinder File Importer
Plugin URI: http://www.aioros.net/splinder-importer-2010/
Description: Import posts, comments and categories from a Splinder blog export file.
Author: Stefano Morciano
Author URI: http://www.aioros.net/
Version: 0.5
License: GPL v3 - http://www.gnu.org/licenses/gpl-3.0.html
*/

if ( !defined( 'WP_LOAD_IMPORTERS' ) )
	return;

// Load Importer API
require_once ABSPATH . 'wp-admin/includes/import.php';

if ( !class_exists( 'WP_Importer' ) ) {
	$class_wp_importer = ABSPATH . 'wp-admin/includes/class-wp-importer.php';
	if ( file_exists( $class_wp_importer ) )
		require_once $class_wp_importer;
}

/**
 * Splinder Importer
 *
 * @package WordPress
 * @subpackage Importer
 */
class Splinder_Import extends WP_Importer {
	var $importer_code = 'splinder-file-importer';
	var $importer_name = 'Splinder';
	var $importer_desc = '';
	var $today;
	var $totalfoundposts = 0;
	var $totalimportedposts = 0;
	var $totalfoundcoms = 0;
	var $totalimportedcoms = 0;
	var $verboseposts = "";
	var $verbosecoms = "";

	/**
	 * Class constructor
	 *
	 * @return void
	 */
	function __construct() {
		$this->today = getdate();
		$this->importer_desc = __( 'Import a Splinder blog', 'splinder-file-importer' );
	}

	/**
	 * PHP 4 style constuctor
	 *
	 * @return void
	 */
	function Splinder_Import() {
		$this->__construct();
	}

	/**
	 * Print header
	 * 
	 * @return void
	 */
	function header() {
		echo '<div class="wrap">';
		screen_icon();
		echo '<h2>' . __( 'Import Splinder', 'splinder-file-importer' ) . '</h2>';
	}

	/**
	 * Print footer
	 * 
	 * @return void
	 */
	function footer() {
		echo '</div>';
	}

	/**
	 * Print greeting
	 * 
	 * @return void
	 */
	function greet() {
		echo '<div class="narrow">';
		$this->import_splinder_form( "admin.php?import=" . $this->importer_code . "&amp;step=1" );
		echo '</div>';
	}
	
	/**
	 * Import calls each step of the import process
	 * 
	 * @param string $blogurl
	 * @param string $start_month
	 * @param string $start_year
	 * @param string $end_month
	 * @param string $end_year
	 * @return void
	 */
	function import( $blogurl, $start_month, $start_year, $end_month, $end_year, $tagsorcats, $uncategorized, $othercat, $verboseposts, $verbosecoms ) {
		$this->verboseposts = $verboseposts;
		$this->verbosecoms = $verbosecoms;
		$month = $start_month;
		$year = $start_year;
		$start_time = time();
		$max_time = ini_get( 'max_execution_time' );
		//while ( ( $year < $end_year ) || ( ( $year == $end_year ) && ( $month <= $end_month ) ) ) {
			//echo time() . " - $start_time<br />max = $max_time<br />";
			/*if ( ($max_time > 0) && (time() - $start_time > 0.7 * $max_time ) ) {
				echo "<p><h3><a href=\"admin.php?import=" . $this->importer_code
					. "&amp;step=2&amp;bu={$blogurl}&amp;sm={$month}&amp;sy={$year}&amp;em={$end_month}&amp;ey={$end_year}"
					. "&amp;tc={$tagsorcats}&amp;un={$uncategorized}&amp;ot={$othercat}"
					. "&amp;vp={$verboseposts}&amp;vc={$verbosecoms}"
					. "\">"
					. __( 'Continue importing...', 'splinder-file-importer' ) . "</a></h3></p>";
				return;
			} else {*/
			
				//echo "<br />" . sprintf( "%04d-%02d", $year, $month ) . "<br />";
				
				libxml_use_internal_errors(true);
				$dom = new DOMDocument("1.0", "UTF-8");
				$dom->strictErrorChecking = false;
				$dom->validateOnParse = false;
				$dom->recover = true;
				
				$xmlstring = file_get_contents($blogurl);
				$xmlstring = iconv("UTF-8","UTF-8//IGNORE", $xmlstring);
				$xmlstring = str_replace("&", "&amp;", $xmlstring);
				$xmlstring = preg_replace("/&amp;(\S+);/", "&$1;", $xmlstring);
				
				$xmlstring = preg_replace('/[\x00-\x08\x10\x0B\x0C\x0E-\x19\x7F]'.
										'|[\x00-\x7F][\x80-\xBF]+'.
										'|([\xC0\xC1]|[\xF0-\xFF])[\x80-\xBF]*'.
										'|[\xC2-\xDF]((?![\x80-\xBF])|[\x80-\xBF]{2,})'.
										'|[\xE0-\xEF](([\x80-\xBF](?![\x80-\xBF]))|(?![\x80-\xBF]{2})|[\x80-\xBF]{3,})/S',
										'?', $xmlstring);

				$xmlstring = preg_replace('/\xE0[\x80-\x9F][\x80-\xBF]'.
										'|\xED[\xA0-\xBF][\x80-\xBF]/S','?', $xmlstring );
				
				$dom->loadXML($xmlstring);
				$dom->save("C:\\xmltest.xml");
				
				$feed = simplexml_import_dom($dom);

				libxml_clear_errors();
				libxml_use_internal_errors(false);

				//$feed = new SimpleXMLElement($blogurl, NULL, TRUE);
				
				foreach ($feed->entry as $post) {
					$post_date = substr(str_replace("T", " ", (string)$post->published), 0, -6);
					$post_year = (int)substr($post_date, 0, 4);
					$post_month = (int)substr($post_date, 5, 2);

					if (($post_year > $start_year || ($post_year == $start_year && $post_month >= $start_month))
					&&($post_year < $end_year || ($post_year == $end_year && $post_month <= $end_month))) {
						echo "<br />" . sprintf( "%04d-%02d", $post_year, $post_month ) . "<br />";
					
						//$body = substr((string)$post->content, 9, -3); // Skip the "<![CDATA[" and "]]>" parts
						$body = (string)$post->content;
						$title = (string)$post->title;
						
						// insert author
						$author = (string)$post->author->name;
						$author_id = $this->add_user( $author );
						
						// insert categories and tags
						$cat_ids = array();
						$tag_ids = array();
						$tag_names = array();
						if ( $tagsorcats == "tags" ) {
							$c = 0;
							foreach ($post->category as $category) {
								$tag_ids[$c] = $this->add_tag( $category['term'] );
								$tag_ids[$c] = $tag_ids[$c]['term_id'];
								$tag_names[$c] = $category['term'];
								$c++;
							}
							if ($c>0) {
								$tag_names = implode( ",", $tag_names );
								if ( $uncategorized == "default" )
									$cat_ids = array( get_option( 'default_category' ) );
								else if ( $uncategorized == "other" )
									$cat_ids = array( $this->add_category( $othercat ) );
							}
						}
						
						if ( $tagsorcats == "cats" ) {
							$c = 0;
							foreach ($post->category as $category) {
								$cat_ids[$c] = $this->add_category( $category['term'] );
								$c++;
							}
							if ($c==0) {
								if ( $uncategorized == "default" )
									$cat_ids = array( get_option( 'default_category' ) );
								else if ( $uncategorized == "other" )
									$cat_ids = array( $this->add_category( $othercat ) );
							}

						}
						
						// insert post
						$post_id = $this->add_post( $title, $body, $author_id, $post_date, $cat_ids, $tag_names );
						
						// insert comments
						foreach ($post->comment as $comment) {
							$com_author = (string)$comment->author->name;
							$com_date = substr(str_replace("T", " ", (string)$comment->published), 0, -6);
							$com_body = (string)$comment->body;
							$comment_id = $this->add_comment( $post_id, $com_body, $com_author, null, $com_date );
						}
						
					}
				}

				/*$month = $month % 12 + 1; // - what, that actually works? - i know, right?
				$year = $year + ( $month == 1 );
				
			}*/
		//}

		echo '<br />';
		echo __( 'Found:', 'splinder-file-importer' ) . ' ' . $this->totalfoundposts . ' ' . __('posts', 'splinder-file-importer');
		echo ', ' . $this->totalfoundcoms . ' ' . __('comments', 'splinder-file-importer');
		echo '<br />';
		echo __( 'Imported:', 'splinder-file-importer' ) . ' ' . $this->totalimportedposts . ' ' . __('posts', 'splinder-file-importer');
		echo ', ' . $this->totalimportedcoms . ' ' . __('comments', 'splinder-file-importer');
		echo '<h3>' . sprintf( __( 'All done. %sHave fun!%s', 'splinder-file-importer' ), "<a href=\"" . get_option( 'home' ) . "\">", "</a>") . '</h3>';
	}

	/**
	 * Convert month name to month number
	 * 
	 * @param string $string
	 * @return int
	 */
	function month_numberize( $string ) {
		$a1 = array( "gennaio", "febbraio", "marzo", "aprile", "maggio", "giugno", "luglio", "agosto", "settembre", "ottobre", "novembre", "dicembre" );
		$a3 = array( 'january', 'february', 'mars', 'april', 'may', 'june', 'july', 'august', 'september', 'october', 'november', 'december' );
		$a2 = array( 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 );
		$string = str_replace( $a1, $a2, strtolower( $string ) );
		return $string = str_replace( $a3, $a2, strtolower( $string ) );
	}

	/**
	 * Convert month number to month name
	 * 
	 * @param int $month
	 * @return string
	 */
	function month_tostring( $month ) {
		$a1 = array( __( 'October', 'splinder-file-importer' ), __( 'November', 'splinder-file-importer' ), __( 'December', 'splinder-file-importer' ),
					__( 'January', 'splinder-file-importer' ), __( 'February', 'splinder-file-importer' ), __( 'March', 'splinder-file-importer' ),
					__( 'April', 'splinder-file-importer' ), __( 'May', 'splinder-file-importer' ), __( 'June', 'splinder-file-importer' ),
					__( 'July', 'splinder-file-importer' ), __( 'August', 'splinder-file-importer' ), __( 'September', 'splinder-file-importer' ) );
		$a2 = array( 10, 11, 12, 1, 2, 3, 4, 5, 6, 7, 8, 9 );
		return str_replace( $a2, $a1, $month );
	}

	/**
	 * Code from IoBloggo importer (http://www.andreabeggi.net/2008/10/30/importare-da-iobloggo-a-wordpress/)
	 * 
	 * @param string $username
	 * @return int
	 */
	function add_user( $username ) {
		$user_id = username_exists( $username );
		if ( !$user_id ) {
			$admin_info = get_userdata( 1 );
			if ( $username == $admin_info->nickname )
				return 1;
			$pass = substr( md5( uniqid( microtime() ) ), 0, 6 );
			$user_id = wp_create_user( $username, $pass );
			printf( __( 'Created user %s', 'splinder-file-importer' ) . "<br />", "<i>$username</i>" );
		}
		return $user_id;
	}

	/**
	 * Add Category
	 * 
	 * @param string $catname
	 * @return int
	 */
	function add_category( $catname ) {
		$category_id = category_exists( $catname );
		if ( !$category_id ) {
			$category_id = wp_create_category( $catname );
			printf( __( 'Created category %s', 'splinder-file-importer' ) . "<br />", "<i>$catname</i>" );
		}
		return $category_id;
	}

	/**
	 * Add Tag
	 * 
	 * @param string $tagname
	 * @return int
	 */
	function add_tag( $tagname ) {
		$tag_id = tag_exists( $tagname );
		if ( !$tag_id ) {
			$tag_id = wp_create_tag( $tagname );
			printf( __( 'Created tag %s', 'splinder-file-importer' ) . "<br />", "<i>$tagname</i>" );
		}
		return $tag_id;
	}

	/**
	 * Add Post
	 * 
	 * @param string $title
	 * @param string $content
	 * @param string $author
	 * @param string $date
	 * @param string $categories
	 * @param string $tags
	 * @return int
	 */
	function add_post( $title, $content, $author, $date, $categories, $tags ) {
		$this->totalfoundposts++;
		$post_id = $this->post_exists( $author, $date, $content );
		if ( !$post_id ) {
			$post = array();
			$post['post_title'] = $title;
			$post['post_content'] = $content;
			$post['post_author'] = $author;
			$post['post_date'] = $date;
			$post['post_category'] = $categories;
			$post['tags_input'] = $tags;
			$post['post_status'] = "publish";
			$post_id = wp_insert_post( $post );
			if ( $this->verboseposts == "posts" ) {
				$userdata = get_userdata($author);
				printf( __( 'Added post %s', 'splinder-file-importer') . "<br />", "<i>$title</i>, " . $userdata->user_login . ", $date" );
			}
			$this->totalimportedposts++;
		}
		return $post_id;
	}

	/**
	 * Add Comment
	 * 
	 * @param int $post_id
	 * @param string $content
	 * @param string $author
	 * @param string $author_url
	 * @param string $date
	 * @return int
	 */
	function add_comment( $post_id, $content, $author, $author_url, $date ) {
		$this->totalfoundcoms++;
		$comment_id = $this->comment_exists( $author, $date, $content );
		if ( !$comment_id ) {
			$comment = array();
			$comment['comment_post_ID'] = $post_id;
			$comment['comment_content'] = $content;
			$comment['comment_author'] = $author;
			$comment['comment_author_url'] = $author_url;
			$comment['comment_date'] = $date;
			$comment = wp_filter_comment( $comment );
			$comment_id = wp_insert_comment( $comment );
			if ( $this->verbosecoms == "comments" )
				printf( __( 'Added comment by %s', 'splinder-file-importer' ) . "<br />", $author . ", $date" );
			$this->totalimportedcoms++;
		}
		return $comment_id;
	}

	/**
	 * Check if a post exists
	 * 
	 * @param string $author
	 * @param string $date
	 * @return bool
	 */
	function post_exists( $author, $date, $content ) {
		global $wpdb;
		$query = $wpdb->prepare( "SELECT * FROM $wpdb->posts WHERE post_author = %d AND post_date = %s", $author, $date );
		$result = $wpdb->get_results( $query );
		foreach ($result as $post) {
			if ($post->post_content == $content) {
				if ( $this->verboseposts == "posts" ) {
					$userdata = get_userdata($author);
					printf( __( "Post by %s already exists", 'splinder-file-importer' ) . "<br />", $userdata->user_login . " (" . $date . ")");
				}
				return $post->ID;
			}
		}
		return FALSE;
	}

	/**
	 * Check if a comment exists
	 * 
	 * @param string $author
	 * @param string $date
	 * @return bool
	 */
	function comment_exists( $author, $date, $content ) {
		global $wpdb;
		$query = $wpdb->prepare( "SELECT * FROM $wpdb->comments WHERE comment_author = %d AND comment_date = %s", $author, $date );
		$result = $wpdb->get_results( $query );
		foreach ($result as $comment) {
			if ($comment->comment_content == $content) {
				if ( $this->verbosecoms == "comments" )
					printf( __( "Comment by %s already exists", 'splinder-file-importer' ) . "<br />", $author . " (" . $date . ")");
				return $comment->comment_ID;
			}
		}
		return FALSE;
	}

	/**
	 * Print the splinder form
	 * 
	 * @param string $action
	 * @return void
	 */
	function import_splinder_form( $action ) {
?>
		<form enctype="multipart/form-data" id="import-form" method="post" action="<?php echo esc_attr( wp_nonce_url( $action, 'import-splinder-form' ) ); ?>">
		<p>
		<?php _e( "Export your Splinder blog (Blogs -> Settings -> Export and Redirect), then right-click the link for the export file, copy the URL and paste it here.", 'splinder-file-importer' ); ?>
		</p><p>
		<label for="import-url"><?php echo __( 'XML File URL:', 'splinder-file-importer' ) . ' '; ?></label>
		<input type="text" id="import-url" name="import-url" size="80" />
		</p><p>
		<label for="start-month"><?php _e( 'From:', 'splinder-file-importer' ); ?></label>
		<select id="start-month" name="start-month">
			<?php
			for ( $i = 1; $i <= 12; $i++ ) {
				echo "<option value=\"$i\"";
				if ( 1 == $i )
					echo ' selected="selected"';
				echo ">" . $this->month_tostring( $i ) . "</option>";
			}
			?>
		</select>
		<input type="text" id="start-year" name="start-year" value="2001" size="5" />
		</p><p>
		<label for="end-month"><?php _e( 'To:', 'splinder-file-importer' ); ?></label>
		<select id="end-month" name="end-month">
			<?php
			for ( $i = 1; $i <= 12; $i++ ) {
				echo "<option value=\"$i\"";
				if ( $i == $this->today['mon'] )
					echo ' selected="selected"';
				echo ">" . $this->month_tostring( $i ) . "</option>";
			}
			?>
		</select>
		<input type="text" id="end-year" name="end-year" value="<?php echo $this->today['year']; ?>" size="5" />
		</p><p>
		<label for="tagsorcats"><?php _e( 'Splinder categories:', 'splinder-file-importer' ); ?></label><br />
		<input type="radio" id="tagsorcats" name="tagsorcats" value="tags" checked="checked" /><?php _e( 'Import as WordPress tags', 'splinder-file-importer' ); ?><br />
		<input type="radio" id="tagsorcats" name="tagsorcats" value="cats" /><?php _e( 'Import as WordPress categories', 'splinder-file-importer' ); ?><br />
		</p><p>
		<label for="uncategorized"><?php _e( 'Posts without a category:', 'splinder-file-importer' ); ?></label><br />
		<input type="radio" id="uncategorized" name="uncategorized" value="default" checked="checked" /><?php _e( 'Assign to my WordPress default category', 'splinder-file-importer' ); ?><br />
		<input type="radio" id="uncategorized" name="uncategorized" value="other" /><?php _e( 'Assign to category...', 'splinder-file-importer' ); ?>
		<input type="text" id="other-category" name="other-category" size="20" />
		</p><p>
		<input type="checkbox" id="verbose-posts" name="verbose-posts" value="posts" checked="checked" /><?php _e( 'List all imported posts', 'splinder-file-importer' ); ?><br />
		<input type="checkbox" id="verbose-comments" name="verbose-comments" value="comments" /><?php _e( 'List all imported comments', 'splinder-file-importer' ); ?><br />
		</p>
		<p class="submit">
		<input type="submit" class="button" value="<?php esc_attr_e( "Start import", 'splinder-file-importer' ); ?>" />
		</p>
		</form>
		<?php
	}

	/**
	 * Callback for register_importer. This function will figure out what to do when the importer is initialized.
	 * 
	 * @return void
	 */
	function dispatch() {
		$step = isset( $_GET['step'] ) ? (int) $_GET['step'] : 0;

		$this->header();
		switch ( $step ) {
			case 0:
				$this->greet();
				break;
			case 1:
				check_admin_referer( 'import-splinder-form' );
				$this->import( $_POST['import-url'],
								$_POST['start-month'], $_POST['start-year'], $_POST['end-month'], $_POST['end-year'],
								$_POST['tagsorcats'], $_POST['uncategorized'], $_POST['other-category'],
								$_POST['verbose-posts'], $_POST['verbose-comments'] );
				break;
			case 2:
				$this->import( $_GET['bu'], $_GET['sm'], $_GET['sy'], $_GET['em'], $_GET['ey'],
								$_GET['tc'], $_GET['un'], $_GET['ot'], $_GET['vp'], $_GET['vc'] );
				break;
		}
		$this->footer();
	}
}

$splinder_import = new Splinder_Import();
register_importer( $splinder_import->importer_code, $splinder_import->importer_name, $splinder_import->importer_desc, array( &$splinder_import, 'dispatch' ) );

function splinder_importer_init() {
	load_plugin_textdomain( 'splinder-file-importer', false, dirname( plugin_basename( __FILE__ ) ) . '/languages' );
}
add_action( 'init', 'splinder_importer_init' );
